@@ -7,12 +7,23 @@ from tuxmake.exceptions import UnsupportedArchitecture
 class Architecture(ConfigurableObject):
     basedir = "arch"
     exception = UnsupportedArchitecture
-    config_aliases = {"aarch64": "arm64", "amd64": "x86_64"}
+    config_aliases = {
+        "aarch64": "arm64",
+        "amd64": "x86_64",
+        "armhf": "arm",
+        "armel": "armv5",
+        "armv8l": "arm",
+        "i686": "i386",
+    }
 
     def __init_config__(self):
         self.targets = self.config["targets"]
         self.artifacts = self.config["artifacts"]
         self.makevars = self.config["makevars"]
+        try:
+            self.dynamic_makevars = self.config["dynamic-makevars"]
+        except KeyError:
+            self.dynamic_makevars = {}
         self.aliases = [k for k, v in self.config_aliases.items() if v == self.name]
         try:
             self.images = self.config["images"]
@@ -37,4 +48,4 @@ class Native(Architecture):
         self.makevars = {}
 
 
-host_arch = Native()
+native_arch = Native()
