@@ -3,27 +3,31 @@ from collections import OrderedDict
 from pathlib import Path
 import json
 import os
+import signal
 import shutil
 import subprocess
-import sys
+import tempfile
 import time
 from tuxmake import __version__
+from tuxmake import deprecated
 from tuxmake.logging import set_debug, debug
-from tuxmake.arch import Architecture, host_arch
+from tuxmake.arch import Architecture, native_arch
 from tuxmake.toolchain import Toolchain, NoExplicitToolchain
 from tuxmake.wrapper import Wrapper
-from tuxmake.output import get_default_build_dir, get_new_output_dir
+from tuxmake.output import get_new_output_dir
+from tuxmake.target import Compression
+from tuxmake.target import default_compression
 from tuxmake.target import create_target
-from tuxmake.runtime import get_runtime
-from tuxmake.metadata import MetadataExtractor
-from tuxmake.exceptions import BuildDirAlreadyExists
+from tuxmake.runtime import Runtime
+from tuxmake.runtime import Terminated
+from tuxmake.metadata import MetadataCollector
 from tuxmake.exceptions import EnvironmentCheckFailed
 from tuxmake.exceptions import UnrecognizedSourceTree
 from tuxmake.exceptions import UnsupportedArchitectureToolchainCombination
 from tuxmake.exceptions import UnsupportedMakeVariable
 from tuxmake.log import LogParser
 from tuxmake.cmdline import CommandLine
-from tuxmake.utils import defaults
+from tuxmake.build_utils import defaults
 from tuxmake.utils import quote_command_line
 from tuxmake.utils import get_directory_timestamp
 
@@ -120,6 +124,8 @@ class Build:
       etc).
     - **targets**: targets to build, list of `str`. If `None` or an empty list
       is passed, the default list of targets will be built.
+    - **compression_type**: compression type to use in compressed artifacts.
+      `str`, must be one of "xz", "none".
     - **kernel_image**: which kernel image to build, overriding the default
       kernel image name defined for the target architecture.
     - **jobs**: number of concurrent jobs to run (as in `make -j N`). `int`,
@@ -159,9 +165,11 @@ class Build:
         kconfig_add=[],
         make_variables={},
         targets=defaults.targets,
+        compression_type=None,
         kernel_image=None,
         jobs=None,
         runtime=None,
+        fail_fast=False,
         verbose=False,
         quiet=False,
         debug=False,
@@ -179,16 +187,13 @@ class Build:
             self.clean_build_tree = True
         self.auto_cleanup = auto_cleanup
 
-        self.target_arch = target_arch and Architecture(target_arch) or host_arch
+        self.target_arch = target_arch and Architecture(target_arch) or native_arch
         self.toolchain = toolchain and Toolchain(toolchain) or NoExplicitToolchain()
         self.wrapper = wrapper and Wrapper(wrapper) or Wrapper("none")
 
         self.timestamp = get_directory_timestamp(self.source_tree)
-        self.environment = {}
-        self.environment["KBUILD_BUILD_TIMESTAMP"] = "@" + self.timestamp
-        self.environment["KBUILD_BUILD_USER"] = "tuxmake"
-        self.environment["KBUILD_BUILD_HOST"] = "tuxmake"
-        self.environment.update(environment)
+        self.__environment__ = None
+        self.__environment_input__ = environment
 
         self.kconfig = kconfig
         self.kconfig_add = kconfig_add
@@ -198,6 +203,8 @@ class Build:
                 raise UnsupportedMakeVariable(k)
         self.make_variables = make_variables
 
+        self.dynamic_make_variables = dict(self.target_arch.dynamic_makevars)
+
         if not targets:
             targets = defaults.targets
 
@@ -206,6 +213,10 @@ class Build:
         else:
             self.target_overrides = self.target_arch.targets
 
+        if compression_type:
+            self.compression = Compression(compression_type)
+        else:
+            self.compression = default_compression
         self.targets = []
         self.__ordering_only_targets__ = {}
         for t in targets:
@@ -218,12 +229,19 @@ class Build:
         else:
             self.jobs = defaults.jobs
 
-        self.runtime = get_runtime(runtime)
+        self.runtime = Runtime.get(runtime)
+        self.runtime.set_image(get_image(self))
         if not self.runtime.is_supported(self.target_arch, self.toolchain):
             raise UnsupportedArchitectureToolchainCombination(
                 f"{self.target_arch}/{self.toolchain}"
             )
 
+        self.fail_fast = fail_fast
+        self.interrupted = False
+        if self.fail_fast:
+            self.keep_going = []
+        else:
+            self.keep_going = ["--keep-going"]
         self.verbose = verbose
         self.quiet = quiet
         self.debug = debug
@@ -231,11 +249,10 @@ class Build:
 
         self.offline = False
 
-        self.artifacts = {"log": ["build.log"]}
-        self.__logger__ = None
+        self.artifacts = {"log": ["build.log", "build-debug.log"]}
         self.__status__ = {}
         self.__durations__ = {}
-        self.metadata_extractor = MetadataExtractor(self)
+        self.metadata_collector = MetadataCollector(self)
         self.metadata = OrderedDict()
         self.cmdline = CommandLine()
 
@@ -251,7 +268,7 @@ class Build:
         return self.__status__
 
     def add_target(self, target_name, ordering_only=False):
-        target = create_target(target_name, self)
+        target = create_target(target_name, self, self.compression)
 
         if ordering_only:
             if target_name not in self.__ordering_only_targets__:
@@ -283,14 +300,33 @@ class Build:
         raise UnrecognizedSourceTree(source.absolute())
 
     def prepare(self):
-        self.log(
-            "# to reproduce this build locally: "
-            + quote_command_line(self.cmdline.reproduce(self))
-        )
         self.wrapper.prepare_host()
-        self.runtime.prepare(self)
+
+        self.runtime.basename = "build"
+        self.runtime.quiet = self.quiet
+        self.runtime.source_dir = self.source_tree
+        self.runtime.output_dir = self.output_dir
+        self.runtime.add_volume(self.build_dir)
+        if self.wrapper.path:
+            self.runtime.add_volume(
+                str(self.wrapper.path), f"/usr/local/bin/{self.wrapper.name}"
+            )
+        wenv = self.wrapper.environment
+        env = dict(**wenv, **self.environment, LANG="C")
+        self.runtime.environment = env
+        for k, v in wenv.items():
+            if k.endswith("_DIR"):
+                self.runtime.add_volume(v)
+        self.runtime.prepare()
         self.wrapper.prepare_runtime(self)
 
+        if self.toolchain.version_suffix and self.runtime.name == "null":
+            toolchain = self.toolchain
+            compiler = toolchain.compiler(self.target_arch)
+            self.log(
+                f"W: Requested {toolchain}, but versioned toolchains are not supported by the null runtime. Will use whatever version of {compiler} that you have installed. To ensure {toolchain} is used, try use a container-based runtime instead."
+            )
+
     @property
     def output_dir(self):
         if self.__output_dir__:
@@ -312,13 +348,23 @@ class Build:
             self.__build_dir__ = Path(self.__build_dir_input__)
             self.__build_dir__.mkdir(parents=True, exist_ok=True)
         else:
-            self.__build_dir__ = get_default_build_dir()
-            try:
-                self.__build_dir__.mkdir()
-            except FileExistsError:
-                raise BuildDirAlreadyExists(self.__build_dir__)
+            self.__build_dir__ = self.output_dir / "build"
+            self.__build_dir__.mkdir()
         return self.__build_dir__
 
+    @property
+    def environment(self):
+        if self.__environment__ is not None:
+            return self.__environment__
+        env = {}
+        env["KBUILD_BUILD_TIMESTAMP"] = "@" + self.timestamp
+        env["KBUILD_BUILD_USER"] = "tuxmake"
+        env["KBUILD_BUILD_HOST"] = "tuxmake"
+        env["KCFLAGS"] = f"-ffile-prefix-map={self.build_dir}/="
+        env.update(self.__environment_input__)
+        self.__environment__ = env
+        return self.__environment__
+
     def get_silent(self):
         if self.verbose:
             return []
@@ -333,7 +379,7 @@ class Build:
         finally:
             self.offline = False
 
-    def run_cmd(self, origcmd, stdout=None, interactive=False):
+    def run_cmd(self, origcmd, stdout=None, interactive=False, echo=True, makevars={}):
         """
         Performs the build.
 
@@ -342,7 +388,7 @@ class Build:
         """
         cmd = []
         for c in origcmd:
-            cmd += self.expand_cmd_part(c)
+            cmd += self.expand_cmd_part(c, makevars)
 
         if cmd[0] == "!":
             expect_failure = True
@@ -350,43 +396,20 @@ class Build:
         else:
             expect_failure = False
 
-        final_cmd = self.runtime.get_command_line(
-            self, cmd, interactive, offline=self.offline
-        )
-        extra_env = dict(**self.wrapper.environment, **self.environment, LANG="C")
-        env = dict(os.environ, **extra_env)
-
-        logger = self.logger.stdin
-        if interactive:
-            stdout = stderr = stdin = None
-        else:
-            stdin = subprocess.DEVNULL
-            stderr = logger
-            if not stdout:
-                self.log(quote_command_line(cmd))
-                stdout = logger
-
-        debug(f"Command: {final_cmd}")
-        if extra_env:
-            debug(f"Environment: {extra_env}")
         try:
             with self.measure_duration("Command"):
-                process = subprocess.Popen(
-                    final_cmd,
-                    cwd=self.source_tree,
-                    env=env,
-                    stdin=stdin,
+                return self.runtime.run_cmd(
+                    cmd,
+                    interactive=interactive,
+                    echo=echo,
                     stdout=stdout,
-                    stderr=stderr,
+                    expect_failure=expect_failure,
+                    offline=self.offline,
                 )
-            process.communicate()
-            if expect_failure:
-                return process.returncode != 0
-            else:
-                return process.returncode == 0
-        except KeyboardInterrupt:
-            process.terminate()
-            sys.exit(1)
+        except (KeyboardInterrupt, Terminated) as ex:
+            self.log(str(ex))
+            self.interrupted = True
+            return False
 
     @contextmanager
     def measure_duration(self, name, metadata=None):
@@ -399,13 +422,14 @@ class Build:
                 self.__durations__[metadata] = duration
             debug(f"{name} finished in {duration} seconds.")
 
-    def expand_cmd_part(self, part):
+    def expand_cmd_part(self, part, makevars):
         if part == "{make}":
             return (
                 ["make"]
                 + self.get_silent()
-                + ["--keep-going", f"--jobs={self.jobs}", f"O={self.build_dir}"]
-                + self.make_args
+                + self.keep_going
+                + [f"--jobs={self.jobs}", f"O={self.build_dir}"]
+                + self.make_args(makevars)
             )
         elif part == "{tar_caf}":
             return [
@@ -417,6 +441,8 @@ class Build:
                 "--clamp-mtime",
                 "-caf",
             ]
+        elif part == "{z}":
+            return self.compression.command
         else:
             return [self.format_cmd_part(part)]
 
@@ -428,29 +454,18 @@ class Build:
             toolchain=self.toolchain.name,
             wrapper=self.wrapper.name,
             kconfig=self.kconfig,
+            z_ext=self.compression.extension,
             **self.target_overrides,
         )
 
-    @property
-    def logger(self):
-        if not self.__logger__:
-            if self.quiet:
-                stdout = subprocess.DEVNULL
-            else:
-                stdout = sys.stdout
-            self.__logger__ = subprocess.Popen(
-                ["tee", str(self.output_dir / "build.log")],
-                stdin=subprocess.PIPE,
-                stdout=stdout,
-            )
-        return self.__logger__
-
     def log(self, *stuff):
-        subprocess.call(["echo"] + list(stuff), stdout=self.logger.stdin)
+        self.runtime.log(*stuff)
 
-    @property
-    def make_args(self):
-        return [f"{k}={v}" for k, v in self.makevars.items() if v]
+    def make_args(self, makevars):
+        # we want to override target makevars with user provided make_variables
+        expanded_makevars = {k: self.format_cmd_part(v) for k, v in makevars.items()}
+        expanded_makevars.update(self.makevars)
+        return [f"{k}={v}" for k, v in expanded_makevars.items() if v]
 
     @property
     def makevars(self):
@@ -461,10 +476,27 @@ class Build:
         mvars.update(self.wrapper.wrap(mvars))
         return mvars
 
+    def get_dynamic_makevars(self):
+        for k, v in self.dynamic_make_variables.items():
+            self.make_variables[k] = self.get_command_output(v).strip()
+
+    def get_command_output(self, cmd):
+        with tempfile.TemporaryFile(mode="w+", encoding="utf-8") as f:
+            self.run_cmd(["sh", "-c", cmd], stdout=f, echo=False)
+            f.seek(0)
+            return f.read().strip()
+
     def build_all_targets(self):
+        skip_all = False
+        self.get_dynamic_makevars()
         for target in self.targets:
             start = time.time()
-            result = self.build(target)
+            if skip_all:
+                result = BuildInfo("SKIP")
+            else:
+                result = self.build(target)
+                if (self.fail_fast and result.failed) or self.interrupted:
+                    skip_all = True
             result.duration = time.time() - start
             self.status[target.name] = result
 
@@ -475,7 +507,7 @@ class Build:
                 return BuildInfo("SKIP")
 
         for precondition in target.preconditions:
-            if not self.run_cmd(precondition, stdout=subprocess.DEVNULL):
+            if not self.run_cmd(precondition, echo=False, stdout=subprocess.DEVNULL):
                 debug(f"Skipping {target.name} because precondition failed")
                 return BuildInfo("SKIP")
 
@@ -483,7 +515,9 @@ class Build:
 
         fail = False
         for cmd in target.commands:
-            if not self.run_cmd(cmd):
+            if not self.run_cmd(
+                cmd, makevars=target.makevars, interactive=cmd.interactive
+            ):
                 fail = True
                 break
 
@@ -504,10 +538,10 @@ class Build:
         return ret
 
     def copy_artifacts(self, target):
-        if not self.status[target.name].passed:
-            return
         self.artifacts[target.name] = []
         for origdest, src in target.find_artifacts(self.build_dir):
+            if not src.exists():
+                continue
             dest = self.output_dir / origdest
             shutil.copy(src, Path(self.output_dir / dest))
             self.artifacts[target.name].append(origdest)
@@ -533,7 +567,7 @@ class Build:
         s = [info.failed for info in self.status.values()]
         return s and True in set(s)
 
-    def extract_metadata(self):
+    def collect_metadata(self):
         self.metadata["build"] = {
             "targets": [t.name for t in self.targets],
             "target_arch": self.target_arch.name,
@@ -560,8 +594,9 @@ class Build:
             "duration": self.__durations__,
         }
         self.metadata["tuxmake"] = {"version": __version__}
+        self.metadata["runtime"] = self.runtime.get_metadata()
 
-        extracted = self.metadata_extractor.extract()
+        extracted = self.metadata_collector.collect()
         self.metadata.update(extracted)
 
     def save_metadata(self):
@@ -574,21 +609,25 @@ class Build:
         parser.parse(self.output_dir / "build.log")
         return parser.errors, parser.warnings
 
-    def terminate(self):
-        self.logger.terminate()
-
     def cleanup(self):
         self.runtime.cleanup()
         if self.clean_build_tree:
             shutil.rmtree(self.build_dir, ignore_errors=True)
 
     def check_environment(self):
-        self.runtime.prepare(self)
+        self.runtime.prepare()
+        self.get_dynamic_makevars()
         cmd = [str(self.runtime.get_check_environment_command())]
         cmd.append(f"{self.target_arch.name}_{self.toolchain.name}")
         cross = self.makevars.get("CROSS_COMPILE")
         if cross:
             cmd.append(cross)
+        if "CROSS_COMPILE_COMPAT" in self.dynamic_make_variables:
+            cross_compat = self.makevars.get("CROSS_COMPILE_COMPAT")
+            if cross_compat:
+                cmd.append(cross_compat)
+            else:
+                cmd.append("")
         result = self.run_cmd(cmd)
         self.cleanup()
         if not result:
@@ -600,33 +639,40 @@ class Build:
         build can be inspected though the `status`, `passed`, and `failed`
         properties.
         """
+        old_sigterm = signal.signal(signal.SIGTERM, Terminated.handle_signal)
+
+        prepared = False
         try:
-            self.metadata_extractor.before_build()
+            self.metadata_collector.before_build()
 
             with self.measure_duration("Input validation", metadata="validate"):
                 self.validate()
 
             with self.measure_duration("Preparation", metadata="prepare"):
                 self.prepare()
+            prepared = True
+            self.log(quote_command_line(self.cmdline.reproduce(self)))
 
             with self.go_offline():
                 with self.measure_duration("Build", metadata="build"):
                     self.build_all_targets()
-
-                with self.measure_duration("Copying Artifacts", metadata="copy"):
-                    for target in self.targets:
-                        self.copy_artifacts(target)
-
-                with self.measure_duration("Metadata Extraction", metadata="metadata"):
-                    self.extract_metadata()
         finally:
+            with self.measure_duration("Copying Artifacts", metadata="copy"):
+                for target in self.targets:
+                    self.copy_artifacts(target)
+
+            with self.measure_duration("Metadata Extraction", metadata="metadata"):
+                if prepared:
+                    self.collect_metadata()
+
             with self.measure_duration("Cleanup", metadata="cleanup"):
-                self.terminate()
                 if self.auto_cleanup:
                     self.cleanup()
 
             self.save_metadata()
 
+            signal.signal(signal.SIGTERM, old_sigterm)
+
 
 def build(**kwargs):
     """
@@ -641,3 +687,28 @@ def build(**kwargs):
     builder = Build(**kwargs)
     builder.run()
     return builder
+
+
+DEFAULT_CONTAINER_REGISTRY = "docker.io"
+
+
+def get_image(build):
+    image = (
+        os.getenv("TUXMAKE_IMAGE")
+        or deprecated.getenv("TUXMAKE_DOCKER_IMAGE", "TUXMAKE_IMAGE")
+        or build.target_arch.get_image(build.toolchain)
+        or build.toolchain.get_image(build.target_arch)
+    )
+    registry = os.getenv("TUXMAKE_IMAGE_REGISTRY", DEFAULT_CONTAINER_REGISTRY)
+    if registry:
+        parts = image.split("/")
+        localhost = parts[0] == "localhost"
+        remotehost = len(parts[0].split(".")) > 1
+        if len(parts) < 3 and not localhost and not remotehost:
+            # only prepend registry if the image name is not already a full
+            # image name.
+            image = registry + "/" + image
+    tag = os.getenv("TUXMAKE_IMAGE_TAG")
+    if tag:
+        image = image + ":" + tag
+    return image
