@@ -90,10 +90,10 @@ class Runtime(ConfigurableObject):
         * `podman-local`: like `docker-local`, but with Podman.
         """
         name = name or DEFAULT_RUNTIME
-        name = "".join([w.title() for w in re.split(r"[_-]", name)]) + "Runtime"
+        clsname = "".join([w.title() for w in re.split(r"[_-]", name)]) + "Runtime"
         try:
             here = sys.modules[__name__]
-            cls = getattr(here, name)
+            cls = getattr(here, clsname)
             return cls()
         except AttributeError:
             raise InvalidRuntimeError(name)
@@ -541,7 +541,7 @@ class DockerRuntime(ContainerRuntime):
             uid = str(os.getuid())
             self.__exec_as_root(["usermod", "-u", uid, self.__user__])
             if self.__group__:
-                gid = str(os.getuid())
+                gid = str(os.getgid())
                 self.__exec_as_root(["groupmod", "-g", gid, self.__group__])
 
     def __exec_as_root(self, cmd):
