@@ -8,6 +8,7 @@ import urllib.request
 from tuxmake import __version__
 from tuxmake.config import ConfigurableObject, split_commands
 from tuxmake.exceptions import InvalidKConfig
+from tuxmake.exceptions import UnsupportedCompression
 from tuxmake.exceptions import UnsupportedTarget
 from tuxmake.exceptions import UnsupportedKconfig
 from tuxmake.exceptions import UnsupportedKconfigFragment
@@ -17,12 +18,59 @@ def supported_targets():
     return Target.supported()
 
 
+class Command(list):
+    interactive = False
+
+
+class compression_types:
+    class xz:
+        command = ["xz", "-T0", "--force", "--keep"]
+        extension = ".xz"
+
+    class none:
+        command = ["true"]
+        extension = ""
+
+
+class Compression:
+    supported: List[str] = [
+        c for c in compression_types.__dict__ if not c.startswith("_")
+    ]
+
+    def __init__(self, ctype=None):
+        if ctype is None:
+            ctype = "xz"
+        try:
+            self.__type__ = getattr(compression_types, ctype)
+        except AttributeError:
+            raise UnsupportedCompression(ctype)
+
+    def format(self, s) -> str:
+        return s.format(z_ext=self.extension)
+
+    @property
+    def name(self) -> str:
+        return self.__type__.__name__
+
+    @property
+    def extension(self) -> str:
+        return self.__type__.extension
+
+    @property
+    def command(self) -> List[str]:
+        return self.__type__.command
+
+
+default_compression = Compression()
+
+
 class Target(ConfigurableObject):
     basedir = "target"
     exception = UnsupportedTarget
 
-    def __init__(self, name, build):
+    def __init__(self, name, build, compression=default_compression):
         self.build = build
+        self.compression = compression
         self.target_arch = build.target_arch
         super().__init__(name)
 
@@ -34,12 +82,24 @@ class Target(ConfigurableObject):
         self.commands = self.__split_cmds__("target", "commands")
         self.kconfig_add = self.__split_kconfigs__()
         try:
-            self.artifacts = self.config["artifacts"]
+            artifacts = dict(self.config["artifacts"])
+            self.artifacts = {}
+            for k, v in artifacts.items():
+                key = self.compression.format(k)
+                value = self.compression.format(v)
+                self.artifacts[key] = value
         except KeyError:
             mapping = self.build.target_overrides
-            key = mapping[self.name]
-            value = self.target_arch.artifacts[self.name].format(**mapping)
-            self.artifacts = {key: value}
+            if mapping and self.name in mapping:
+                key = mapping[self.name]
+                value = self.target_arch.artifacts[self.name].format(**mapping)
+                self.artifacts = {key: value}
+            else:
+                self.artifacts = {}
+        try:
+            self.makevars = dict(self.config["makevars"])
+        except KeyError:
+            self.makevars = {}
 
     def __str__(self):
         return self.name
@@ -49,7 +109,12 @@ class Target(ConfigurableObject):
 
     def __split_cmds__(self, section, item):
         s = self.config[section].get(item)
-        return split_commands(s)
+        return [Command(c) for c in split_commands(s)]
+
+    def add_command(self, cmd):
+        c = Command(cmd)
+        self.commands.append(c)
+        return c
 
     def prepare(self):
         pass
@@ -85,7 +150,11 @@ class Config(Target):
         conf = self.build.kconfig
         if config.exists():
             return
-        if self.handle_url(config, conf) or self.handle_local_file(config, conf):
+        if (
+            self.handle_url(config, conf)
+            or self.handle_local_file(config, conf)
+            or self.handle_in_tree_file(config, conf)
+        ):
             self.build.log(f"# {conf} -> {config}")
             olddefconfig = True
         elif self.handle_make_target(conf):
@@ -104,16 +173,19 @@ class Config(Target):
             if (
                 self.handle_url(fragfile, frag)
                 or self.handle_local_file(fragfile, frag)
+                or self.handle_in_tree_file(fragfile, frag)
                 or self.handle_inline_fragment(fragfile, frag)
             ):
                 merge.append(str(fragfile))
                 self.build.log(f"# {frag} -> {fragfile}")
             elif self.handle_in_tree_config(frag):
                 pass
+            elif self.handle_explicit_make_target(frag):
+                pass
             else:
                 raise UnsupportedKconfigFragment(frag)
         if merge:
-            self.commands.append(
+            self.add_command(
                 [
                     "scripts/kconfig/merge_config.sh",
                     "-m",
@@ -125,7 +197,7 @@ class Config(Target):
             )
             olddefconfig = True
         if olddefconfig:
-            self.commands.append(["{make}", "olddefconfig"])
+            self.add_command(["{make}", "olddefconfig"])
 
     def handle_url(self, config, url):
         if not url.startswith("http://") and not url.startswith("https://"):
@@ -150,16 +222,37 @@ class Config(Target):
             f.write(path.read_text())
         return True
 
+    def handle_in_tree_file(self, config, filename):
+        path = self.build.source_tree / filename
+        if not path.exists():
+            return False
+
+        with config.open("w") as f:
+            f.write(path.read_text())
+        return True
+
     def handle_make_target(self, t):
         if re.match(r"^[\w\-]+config$", t):
-            self.commands.append(["{make}", t])
+            self.add_command(["{make}", t])
+            return True
+        else:
+            return False
+
+    def handle_explicit_make_target(self, t):
+        if re.match(r"^make:.*$", t):
+            target = re.sub(r"^make:", "", t)
+            self.add_command(["{make}", target])
+            return True
+        elif re.match(r"^imake:.*$", t):
+            target = re.sub(r"^imake:", "", t)
+            self.add_command(["{make}", target]).interactive = True
             return True
         else:
             return False
 
     def handle_in_tree_config(self, t):
         if re.match(r"^[\w\-]+.config$", t):
-            self.commands.append(["{make}", t])
+            self.add_command(["{make}", t])
             return True
         else:
             return False
@@ -193,6 +286,6 @@ class Kernel(Target):
 __special_targets__ = {"config": Config, "kernel": Kernel}
 
 
-def create_target(name, build):
+def create_target(name, build, compression=default_compression):
     cls = __special_targets__.get(name, Target)
-    return cls(name, build)
+    return cls(name, build, compression)
